#!/etc/opt/openit/perl

BEGIN {
    push @INC, "/opt/openit/contrib/lib";
}


use Date::Calc;

use OpeniT::Contrib::Config;

my $config = new OpeniT::Contrib::Config; # Reads default config file(s)

my $datadir = $config->{DATA_DIR};

my @data_types = ( 24, 29, 40, 59 );

my @jobs = ();

foreach my $data_type( @data_types ) {
    if( ! -d "$datadir/$data_type" ) {
	next;
    }
    my $resolution = &_find_resolution( $data_type );
    &_add_jobscheduler_job( $data_type, $resolution );
}

foreach my $job( @jobs ) {
    print "$job\n";
}

my $jobs_str = join "; ", @jobs;
system( $jobs_str );


sub _find_resolution {
    my( $data_type ) = @_;
    my @newest = ();
    my @files = <$datadir/$data_type/M/*/*/*/*/data*>;
    my $sample = &_newest_data( @files );
    if( defined $sample ) {
	push @newest, $sample;
    }
    @files = <$datadir/$data_type/M/*/*/*/data*>;
    my $sample = &_newest_data( @files );
    if( defined $sample ) {
	push @newest, $sample;
    }
    @files = <$datadir/$data_type/W/*/*/data*>;
    my $sample = &_newest_data( @files );
    if( defined $sample ) {
	push @newest, $sample;
    }
    if( scalar @newest == 0 ) {
	warn "Could not find resolution of data type $data_type\n";
	return undef;
    }

    @sorted_resolutions = # We need the highest resolutions of the last day
	sort { ( $$a[ 0 ] <=> $$b[ 0 ] ) || ( $$b[ 1 ] <=> $$a[ 1 ] ) } @newest;

    my $resolution = $sorted_resolutions[ $#sorted_resolutions ][ 1 ];
    return $resolution;
}

sub _newest_data {
    my( @files ) = @_;
    my @samples = ();
    foreach my $file( @files ) {
	my $sample = &_get_date_entry( $file );
	if( defined $sample ) {
	    push @samples, $sample;
	}
    }
    my @sorted = sort { $$a[ 0 ] <=> $$b[ 0 ] } @samples;
    if( scalar @sorted == 0 ) {
	return undef;
    }
    return $sorted[ $#sorted ];
}

sub _get_date_entry {
    my( $file ) = @_;
    if( $file =~ m/\/M\/(\d+)\/(\d+)\/(\d+)\/(\d+)\/data(.gz)?$/ ) {
	return [ sprintf( "%04d%02d%02d", $1, $2, $3 ), $4 ];
    }
    if( $file =~ m/\/M\/(\d+)\/(\d+)\/(\d+)\/data(.gz)?$/ ) {
        return[ sprintf( "%04d%02d%02d", $1, $2, $3 ), 86400 ];
    }
    if( $file =~ m/\/W\/(\d+)\/(\d+)\/data(.gz)?$/ ) {
	my( $year, $week ) = ( $1, $2 );
	my @monday = Date::Calc::Monday_of_Week( $week, $year );
	my $monday_str = sprintf( "%04d%02d%02d", @monday );
	return [ $monday_str, 604800 ];
    }
    return undef;
}


sub _add_jobscheduler_job {
    my( $data_type, $resolution ) = @_;
    my $add = "/opt/openit/bin/add";
    my $cmd = "$add --redo --resolution $resolution --datatype $data_type";
    push @jobs, $cmd;
}
